// 002 u_c_sorted_listview
// 30 oct 2019

// -- (C) Felix John COLIBRI 2019
// -- documentation: http://www.felix-colibri.com

(*
taRightJustify;
*)

(*$r+*)

unit u_c_sorted_listview;
  interface
    uses Classes, ComCtrls;

    type t_sort_kind=  (e_not_sorted, e_ascending, e_descending);
         t_column_info=
             record
               m_column_type: (e_unknown_type, e_string_type, e_integer_type, e_double_type);
               m_sort_kind: t_sort_kind;
             end;

         c_sorted_listview=
             class(tListView)
               public
                 m_name: String;
                 m_total_width: Integer;

                 m_clicked_column: Integer;
                 m_column_info_array: array of t_column_info;

                 m_c_result_list: tStringList;

                 Constructor create_sorted_listview(p_name: String;
                     p_c_owner: tComponent);
                 procedure add_column(p_title: String; p_width: Integer; p_type: String);
                 procedure add_value(p_value_array: Array of String);

                 procedure handle_listview_compare(Sender: TObject;
                     Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
                 // -- dispatch handler
                 procedure ColClick(Column: TListColumn); Override;
                 procedure Click; Override;

                 Destructor Destroy; Override;
             end; // c_sorted_listview

  implementation
    uses SysUtils, Controls
        , u_c_display;

    // -- c_sorted_listview

    Constructor c_sorted_listview.create_sorted_listview(p_name: String;
        p_c_owner: tComponent);
      begin
        Inherited Create(p_c_owner);
        m_name:= p_name;

        Parent:= p_c_owner as tWinControl;
        Align:= alClient;
        ViewStyle:= vsReport;

        HideSelection:= False;
        MultiSelect:= True;

        OnCompare:= handle_listview_compare;
      end; // create_sorted_listview

    procedure c_sorted_listview.add_column(p_title: String; p_width: Integer; p_type: String);
      var l_info_array_index: Integer;
      begin
        with Columns.Add  do
        begin
          if (Length(p_title)> 0) and (p_title[1]= '-')
            then begin
                System.Delete(p_title, 1, 1);
                Caption:= p_title;
                Alignment:= taRightJustify;
              end
            else Caption:= p_title;

          if p_width< 0
            then begin
                Width:= Self.Width- 15;
              end
            else begin
                Width:= p_width;
                Inc(m_total_width, p_width);
              end;
        end; // with file_listview_

        l_info_array_index:= Length(m_column_info_array);
        SetLength(m_column_info_array, l_info_array_index+ 1);

        with m_column_info_array[l_info_array_index] do
          if p_type= ''
            then m_column_type:= e_string_type
            else
              if p_type= 'i'
                then m_column_type:= e_integer_type
                else
                  if p_type= 'd'
                    then m_column_type:= e_double_type;
      end; // add_column

    procedure c_sorted_listview.add_value(p_value_array: Array of String);
      var l_value_index: Integer;
      begin
        with Items.Add do
        begin
          if High(p_value_array)> 0
            then Caption:= p_value_array[0];

          for l_value_index:= 1 to High(p_value_array) do
            SubItems.Add(p_value_array[l_value_index]);
        end; // with Items.Add 
      end; // add_value

    procedure c_sorted_listview.handle_listview_compare(Sender: TObject;
        Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
      var l_direction: Integer;
          l_size_1, l_size_2: Integer;
          l_item_1, l_item_2: String;
      begin
        if m_column_info_array[m_clicked_column].m_sort_kind= e_ascending
          then l_direction:= 1
          else l_direction:= -1;

        if m_clicked_column= 0
          then begin
              l_item_1:= Item1.Caption;
              l_item_2:= Item2.Caption;
            end
          else begin
              l_item_1:= Item1.SubItems[m_clicked_column- 1];
              l_item_2:= Item2.SubItems[m_clicked_column- 1];
            end;

          case m_column_info_array[m_clicked_column].m_column_type of
            e_string_type :
                Compare:= l_direction* CompareText(l_item_1, l_item_2);
            e_integer_type :
                begin
                  l_size_1:= StrToInt(l_item_1);
                  l_size_2:= StrToInt(l_item_2);
                  if l_size_1< l_size_2
                    then Compare:= -1
                    else Compare:= 1;
                  Compare := l_direction* Compare;
                end;
            e_double_type:
                begin
                end;
          end; // case
      end; // ListView1Compare

    procedure c_sorted_listview.ColClick(Column: TListColumn);
      var l_direction: string;
      begin
        // g_c_clicked_column:= Column;
        m_clicked_column:= Column.Index;

        if m_column_info_array[m_clicked_column].m_sort_kind in [e_not_sorted, e_descending]
          then begin
              m_column_info_array[m_clicked_column].m_sort_kind:= e_ascending;
              l_direction:= 'asc';
            end
          else begin
              m_column_info_array[m_clicked_column].m_sort_kind:= e_descending;
              l_direction:= 'desc';
            end;

        display(Format('sort %d %s', [m_clicked_column, l_direction]));

        // file_listview_.SortType:= stText;
        AlphaSort;
      end; // ColClick

    procedure c_sorted_listview.Click;
      var l_list_index: Integer;
          l_selected_count: Integer;

          l_display, l_previous_display: String;
          l_file_name, l_previous_file_name: String;
      begin
        m_c_result_list.Free;
        m_c_result_list:= tStringList.Create;

        l_file_name:= '';
        l_previous_file_name:= '';
        l_previous_display:= '';

        with Items do
        begin
          // l_total_size:= 0;
          l_selected_count:= 0;
          for l_list_index:= 0 to Count- 1 do
            with Items[l_list_index] do
            begin
              l_file_name:= Caption;
              l_display:= Caption;

              m_c_result_list.Add(l_display);

              if Selected
                then begin
                    Inc(l_selected_count);
                  end;

              l_previous_file_name:= l_file_name;
              l_previous_display:= l_display;
            end;
        end; // with file_listview_, Items

        with m_c_result_list do
        begin
          m_c_result_list.Insert(0, Format('// count %d  sel %d', [Items.Count, l_selected_count]));
        end; // with m_c_result_list
      end; // Click

    Destructor c_sorted_listview.Destroy;
      begin
        m_c_result_list.Free;
        Inherited;
      end; // Destroy

    begin // u_c_sorted_listview
    end. // u_c_sorted_listview


